@@ -478,6 +478,11 @@ void setosize(xcWidget w, objinstptr dataptr)
    objinstptr nsobj;
    int res = sscanf(_STR2, "%f", &tmpres);
 
+   // Negative values are flips---deal with them independently
+
+   if (tmpres < 0)
+      tmpres = -tmpres;
+
    if (res == 0 || tmpres <= 0) {
       Wprintf("Illegal value");
       return;
@@ -487,7 +492,7 @@ void setosize(xcWidget w, objinstptr dataptr)
       if (SELECTTYPE(osel) == OBJINST) {
 	 nsobj = SELTOOBJINST(osel);
 	 oldsize = nsobj->scale;
-         nsobj->scale = tmpres;
+         nsobj->scale = (oldsize < 0) ? -tmpres : tmpres;
 
          if (oldsize != tmpres) {
             register_for_undo(XCF_Rescale, UNDO_MORE, areawin->topinstance,
@@ -616,6 +621,7 @@ void setcolorscheme(Boolean boolvalue)
    INFOLABELCOLOR = appdata.infocolor;
    RATSNESTCOLOR = appdata.ratsnestcolor;
    BBOXCOLOR = appdata.bboxpix;
+   CLIPMASKCOLOR = appdata.clipcolor;
 }
 
 /*----------------------------------------------------------------*/
@@ -898,6 +904,8 @@ void setallstylemarks(u_short styleval)
 		(styleval & OPAQUE) ? "opaque" : "transparent");
    XcInternalTagCall(xcinterp, 3, "border", "bbox", (styleval & BBOX) ? "true" :
 		"false");
+   XcInternalTagCall(xcinterp, 3, "border", "clipmask", (styleval & CLIPMASK) ?
+		"true" : "false");
    XcInternalTagCall(xcinterp, 2, "border", (styleval & UNCLOSED) ? "unclosed" :
 		"closed");
    XcInternalTagCall(xcinterp, 2, "border", bptr);
@@ -1003,7 +1011,9 @@ int setelementstyle(xcWidget w, u_short value, u_short mask)
 	       easydraw(*sstyle, DOFORALL);
 
 #ifdef TCL_WRAPPER
-	       register_for_undo(XCF_ChangeStyle, UNDO_DONE, areawin->topinstance,
+	       register_for_undo(XCF_ChangeStyle,
+			(sstyle == areawin->selectlist + areawin->selects - 1) ?
+			UNDO_DONE : UNDO_MORE, areawin->topinstance,
 			SELTOGENERIC(sstyle), (int)oldstyle);
 #else
 	       /* Tcl version differs in that the element is not deselected after */
@@ -1025,6 +1035,11 @@ int setelementstyle(xcWidget w, u_short value, u_short mask)
 	 newstyle &= ~(BBOX);
 	 return -1;
       }
+      else if (value & CLIPMASK) {
+	 Wprintf("Cannot set default style to Clip Mask");
+	 newstyle &= ~(CLIPMASK);
+	 return -1;
+      }
       else {
 	 newstyle &= ~mask;
 	 newstyle |= value;
@@ -1057,7 +1072,7 @@ int setelementstyle(xcWidget w, u_short value, u_short mask)
 void setcolor(xcWidget w, int cindex)
 {
    short *scolor;
-   int *ecolor, cval;
+   int *ecolor, cval, oldcolor;
    Boolean selected = False;
    stringpart *strptr, *nextptr;
 
@@ -1090,9 +1105,14 @@ void setcolor(xcWidget w, int cindex)
       for (scolor = areawin->selectlist; scolor < areawin->selectlist
 	   + areawin->selects; scolor++) {
 	 ecolor = &(SELTOCOLOR(scolor));
-
+	 oldcolor = *ecolor;
 	 *ecolor = cval;
 	 selected = True;
+
+	 register_for_undo(XCF_Color, (scolor == areawin->selectlist
+			+ areawin->selects - 1) ?  UNDO_DONE : UNDO_MORE,
+			areawin->topinstance,
+			SELTOGENERIC(scolor), (int)oldcolor);
       }
    }
 
